/*$Id: trap.c,v 1.4 2002/07/24 10:07:47 aa5779 Exp $*/
/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <signal.h>
#include <string.h>
#include "trap.h"

static volatile sig_atomic_t sig_occured = -1;

static void handler(int signo)
{
	sig_occured = signo;
}

typedef struct sig_name
{
	char *name;
	int num;
} sig_name;

static sig_name sig_names[] = {{"SIGINT", SIGINT}, {"SIGQUIT", SIGQUIT}, 
							   {"SIGTSTP", SIGTSTP}, {"SIGTTIN", SIGTTIN},
							   {"SIGTTOU", SIGTTOU}, {"SIGUSR1", SIGUSR1},
							   {"SIGUSR2", SIGUSR2}, {"SIGALRM", SIGALRM},
							   {"SIGPIPE", SIGPIPE}, {"SIGCHLD", SIGCHLD},
							   {"SIGHUP", SIGHUP}, {NULL}
							  };

void set_trap(char *name, int behaviour)
{
	struct sigaction current;
	sig_name *iter;
	for(iter = sig_names; iter->name; iter++)
	{
		if(!strcmp(iter->name, name))
			break;
	}
	if(!iter->name)
		return;
	current.sa_handler = (behaviour == trap_ignore ? SIG_IGN :
							(behaviour == trap_default ? SIG_DFL : handler)
						 );
	current.sa_flags = SA_RESTART;
	sigemptyset(&current.sa_mask);
	sigaction(iter->num, &current, NULL);
}

char *check_for_trap(void)
{
	sigset_t current, old;
	sig_name *iter;
	if(sig_occured < 0)
		return NULL;
	sigfillset(&current);
	sigprocmask(SIG_SETMASK, &current, &old);
	for(iter = sig_names; iter->name; iter++)
	{
		if(iter->num == sig_occured)
			break;
	}
	sig_occured = -1;
	sigprocmask(SIG_SETMASK, &old, NULL);
	return iter->name;
}
